using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace SpiDotNet
{
    class SpiCmT35 : ISpiInterface
    {
        private IntPtr hDrv = IntPtr.Zero;

        // Device CreateFile specific constants
        internal const Int32 INVALID_HANDLE_VALUE = -1;
        internal uint IOCTL_SPI_CONFIGURE = NativeMethods.CTL_CODE_DEVICE_UNKNOWN(0x0200);
        internal uint IOCTL_SPI_WRITEREAD = NativeMethods.CTL_CODE_DEVICE_UNKNOWN(0x0201);

        [StructLayout(LayoutKind.Sequential)]
        internal class IoctlSpiConfigInfo
        {
            public IoctlSpiConfigInfo(uint address, uint config)
            {
                this.addr = address;
                this.conf = config;
            }
            public uint addr;
            public uint conf;
        }

        public SpiCmT35(uint num) 
        {
            string driverName = "SPI" + num + ":";
            //First, we create a handle to the SPI driver.
            hDrv = NativeMethods.CreateFile(driverName, 0, 0, IntPtr.Zero, 0, 0, IntPtr.Zero);
            if (hDrv.ToInt32() == INVALID_HANDLE_VALUE)
                throw new InvalidOperationException("Can't open " + driverName);
        }

        ~SpiCmT35()
        {
            NativeMethods.CloseHandle(hDrv);
        }
        public bool SPIConfigure(uint address, uint config) 
        {
            IoctlSpiConfigInfo inputParameters = new IoctlSpiConfigInfo(address, config);
            return NativeMethods.SendIOCTL(hDrv, IOCTL_SPI_CONFIGURE, inputParameters, (uint)Marshal.SizeOf(inputParameters));
        }
        public bool SPIWriteRead(uint size, ref uint[] pOutBuffer, uint[] pInBuffer)
        {
            UInt32 inVal = pInBuffer[0];
            UInt32 retVal = pOutBuffer[0];
            NativeMethods.SendSpiIOCTL(hDrv, IOCTL_SPI_WRITEREAD, inVal, (uint)Marshal.SizeOf(inVal), out retVal, (uint)Marshal.SizeOf(retVal));
            pOutBuffer[0] = retVal;
            return true;
        }
    }
}
